// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#if !defined(_IW_GAME_RESOURCES_H_)
#define _IW_GAME_RESOURCES_H_


#include "s3efile.h"
#include "IwGameString.h"
#include "IwGameUtil.h"
#include "IwGameXoml.h"

class CIwGameAnimManager;
class CIwGameImageManager;
class CIwGameResourceGroupManager;
class CIwGameAnimTimelineManager;

// 
// 
//
//
// CIwGameResourceGroup - Wrapper class to allow Marmalade resource groups to be used as game resources
// 
// 
// 
//
class CIwGameResourceGroup : public IIwGameXomlResource
{
	// Properties
protected:
	CIwGameResourceGroupManager*	Parent;					// Parent resource group manager that allocated us
	CIwGameString					GroupName;				// Resource group name
	CIwGameString					GroupFilename;			// Resource group file name
	CIwResGroup*					ResourceGroup;			// Marmalade resource group
	bool							Loaded;					// Loaded state of resource group
public:
	void							setParent(CIwGameResourceGroupManager* parent)	{ Parent = parent; }
	CIwGameResourceGroupManager*	getParent()										{ return Parent; }
	void							setGroupName(const char* group_name)			{ GroupName = group_name; }
	CIwGameString&					getGroupName()									{ return GroupName; }
	void							setGroupFilename(const char* group_file_name)	{ GroupFilename = group_file_name; }
	CIwGameString&					getGroupFilename()								{ return GroupFilename; }
	CIwResGroup*					getResourceGroup();
	// Properties End

public:
	CIwGameResourceGroup() : Parent(NULL), ResourceGroup(NULL), Loaded(false) { setClassType("ResourceGroup"); }
	~CIwGameResourceGroup() { Destroy(); }

	bool							Load();
	void							Destroy();

	// Implementation of IIwGameXomlClass interface
	bool							LoadFromXoml(IIwGameXomlResource* parebt, bool load_children, CIwGameXmlNode* node);
};

//
//
//
//
//  CIwGameResourceGroupCreator - Creates an instance of a resource group ovject
//
//
//
//
class CIwGameResourceGroupCreator : public IIwGameXomlClassCreator
{
public:
	CIwGameResourceGroupCreator()
	{
		setClassName("ResourceGroup");
	}
	IIwGameXomlResource*	CreateInstance(IIwGameXomlResource* parent);
};


// 
// 
//
//
// CIwGameResourceGroupManager - CIwGameResourceGroupManager manages a collection of resource groups
// 
// 
// 
//
class CIwGameResourceGroupManager
{
public:
	// Public access to class creator iteration
	typedef CIwList<CIwGameResourceGroup*>::iterator _Iterator;
	_Iterator				begin() { return Groups.begin(); }
	_Iterator				end() { return Groups.end(); }

protected:
	// Properties
	CIwList<CIwGameResourceGroup*>	Groups;			// A collection of resources groups
public:
	// Properties end

public:
	CIwGameResourceGroupManager() {}
	~CIwGameResourceGroupManager() { clearGroups(); }

	void					addGroup(const char* group_name, const char* group_filename, bool preload = false);
	void					addGroup(CIwGameResourceGroup* group);
	void					removeGroup(CIwGameResourceGroup* group);
	void					removeGroup(unsigned int name_hash);
	CIwGameResourceGroup*	findGroup(unsigned int name_hash, bool global_search = true);
	CIwGameResourceGroup*	findGroup(const char* name, bool global_search = true);
	void					clearGroups();

};

// 
// 
//
//
// CIwGameGlobalResources - Manages application global resources
// 
// 
// 
//
class CIwGameGlobalResources
{
	CDEFINE_SINGLETON(CIwGameGlobalResources)

protected:
	// Properties
	CIwGameImageManager*			ImageManager;			// Manages the allocation and clean up of images
	CIwGameResourceGroupManager*	ResourceGroupManager;	// Manages resource groups
	CIwGameAnimManager*				AnimManager;			// Animation manager
	CIwGameAnimTimelineManager*		TimelineManager;		// Timeline manager
public:
	CIwGameImageManager*			getImageManager()					{ return ImageManager; }
	CIwGameResourceGroupManager*	getResourceGroupManager()			{ return ResourceGroupManager; }
	CIwGameAnimManager*				getAnimManager()					{ return AnimManager; }
	CIwGameAnimTimelineManager*		getTimelineManager()				{ return TimelineManager; }
	// Properties end

public:
	void					Init();
	void					Release();
};
#define IW_GAME_GLOBAL_RESOURCES		CIwGameGlobalResources::getInstance()


#endif	// _IW_GAME_RESOURCES_H_
